# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.api.report`` is a module implementing the report command.

This module contains the logic to examine your source code, extract internal
and external imports, and finally determine which external PyPI packages
you need to install in order to satisfy dependencies.
"""
from __future__ import absolute_import, print_function

import os
import json
import pkgutil

from .. import __url__, stdlibfile, pypifile, libdir
from ..core.logger import logger
from ..core.imports import find_imports
from ..core.util import (find_files, list_files, is_valid_path,
                         custom_sys_path, remove_sys_modules, fill_with_local,
                         fill_with_stdlib, fill_with_pypi)

from setuptools import find_packages

try:
    raw_input
except NameError:
    raw_input = input


def get_package_dirs(path):
    """
    List directories containing python packages on ``path``.

    :param path: a path pointing to a directory containing python code.
    :return: a list containing directories of packages.

    .. versionadded:: 0.1.0
    """
    package_dirs = []
    logger.info('Searching for package directories ...')
    for init in find_files(path, '__init__.py'):
        pkgdir = os.path.dirname(init)
        if os.path.commonprefix([pkgdir, path]) == path and \
           is_valid_path(os.path.relpath(pkgdir, path)):
            while True:
                init = os.path.split(init)[0]
                if not os.path.isfile(os.path.join(init, '__init__.py')):
                    break
            if init not in package_dirs:
                package_dirs.append(init)
    return package_dirs


def get_packages(path):
    """
    List packages living in ``path`` with its directory.

    :param path: a path pointing to a directory containing python code.
    :return: a list of tuples containing the name of the package and
             the package directory. For example::

                 [
                    ('package_a', '/path/to/package_a'),
                    ('package_b.module_b', '/path/to/package_b/module_b'),
                    ('package_c.module_c', '/path/to/package_c/module_c')
                 ]

    .. versionadded:: 0.1.0
    """
    packages = []
    package_dirs = get_package_dirs(path)
    logger.info('Extracting package names from directories ...')

    for _dir in package_dirs:
        for pkgname in find_packages(_dir):
            try:
                with custom_sys_path([_dir, libdir]):
                    with remove_sys_modules([pkgname]):
                        pkgdir = pkgutil.get_loader(pkgname).filename
            except:
                pkgdir = os.path.join(_dir, os.sep.join(pkgname.split('.')))
            logger.debug('Found "{0}" package in '
                         '"{1}".'.format(pkgname, pkgdir))
            packages.append([pkgname, pkgdir])
    return packages


def get_modules(pkgdata):
    """
    List modules inside packages provided in ``pkgdata``.

    :param pkgdata: a list of tuples containing the name of a package and
                    the directory where its located.
    :return: a list of the modules according to the list of packages
             provided in ``pkgdata``.

    .. versionadded:: 0.1.0
    """
    modules = []
    logger.info('Extracting module names from packages ...')

    for pkgname, pkgdir in pkgdata:
        for py in list_files(pkgdir, '*.py'):
            module = os.path.splitext(os.path.basename(py))[0]
            if not module.startswith('__'):
                modname = '.'.join([pkgname, module])
            else:
                modname = pkgname
            logger.debug('Found "{0}" module in '
                         '"{1}" package.' .format(modname, pkgname))
            modules.append(modname)
    return sorted(list(set(modules)))


def get_imports(pkgdata):
    """
    List modules imported inside of packages provided in ``pkgdata``.

    :param pkgdata: a list of tuples containing the name of a package and
                    the directory where its located.
    :return: a list of the modules imported according to the list of packages
             provided in ``pkgdata``.

    .. versionadded:: 0.1.0
    """
    imports = []
    logger.info('Extracting imported modules from packages ...')

    for package, path in pkgdata:
        for filename in list_files(path, '*.py'):
            for i in find_imports(package, filename):
                logger.debug('Found import to "{0}" in "{1}"'
                             ' package.'.format(i, package))
                imports.append(i)
    return imports


def get_module_datadict(basedir):
    """
    Process the current directory to get data from packages and modules.

    :param basedir: a string containing a path to the directory to be analized.
    :return: a dictionary containing information for each imported module.
             Like::

                {
                    'module_a': ['LOCAL'],
                    'module_b': ['pypi_package_1'],
                    'module_c': ['pypi_package_1', 'pypi_package_2'],
                    'module_d': [],
                    'module_e': ['STDLIB2.6', 'STDLIB2.7', 'STDLIB3.5'],
                    'module_f': ['STDLIB2.7'],
                }

    .. versionadded:: 0.1.0
    """
    with open(stdlibfile, 'r') as s:
        stdlibdata = json.loads(s.read())

    with open(pypifile, 'r') as p:
        pypidata = json.loads(p.read())

    packages = get_packages(basedir)
    modules = get_modules(packages)
    imports = get_imports(packages)

    datadict = dict((m, []) for m in imports)
    datadict = fill_with_local(datadict, modules)
    datadict = fill_with_stdlib(datadict, stdlibdata)
    datadict = fill_with_pypi(datadict, pypidata)

    return datadict


def ask_multiple_pypi(datadict):
    """
    Ask the user about which PyPI package will use to satisfy an import.

    :param datadict: a dictionary containing modules as keys and
                     a list as values.
    :return: an updated ``datadict`` with the answered information from
             user.

    .. versionadded:: 0.1.0
    """
    for module, where in datadict.items():
        if len(where) < 2 or 'LOCAL' in where or \
           any('STDLIB' in s for s in where):
            continue

        print(('There is more than one PyPI package that satisfies'
               ' this module: {0}').format(module))

        while True:
            print('\nPlease write the one you would like to use.')
            for w in where:
                print('    - {0}'.format(w))

            selected = raw_input('\n>> ')
            if selected not in where:
                print('"{0}" not available.'.format(selected))
                continue

            datadict[module] = [selected]
            break

    return datadict


def get_messages(datadict):
    """
    Generate messages for each type of module in ``datadict``.

    :param datadict: a dictionary containing modules as keys and
                     a list as values.
    :return: a dictionary containing messages for each type of module.

    .. versionadded:: 0.1.0
    """
    msg = {'l': [], 's': [], 'n': [], 'p': []}
    for module, where in datadict.items():
        if 'LOCAL' in where:
            msg['l'].append(module)
        elif any('STDLIB' in s for s in where):
            msg['s'].append('{0}:{1}'.format(module, ','.join(sorted(where))))
        elif not where:
            msg['n'].append(module)
        else:
            msg['p'].append('{0}:{1}'.format(module, where[0]))
    return msg


def main(**kwargs):
    """
    Generate a report to inform about PyPI dependencies.

    .. _PyPIContents: https://github.com/LuisAlejandro/pypicontents

    This command will search your code for unsatisfied dependencies by
    looking at your ``import`` statements. If an import is not satisfied by
    internal modules or the standard library, then it will query the
    PyPI module index provided by `PyPIContents`_.

    Sometimes, more than one PyPI package will provide the missing module and
    in such cases, you will be asked to select one from a list of options.

    If Pip Sala Bim fails to find a package providing the module you need, it
    will report it back to you.

    :return: an exit status.

    .. versionadded:: 0.1.0
    """
    basedir = os.getcwd()
    requirements = kwargs.get('requirements', False)

    if not os.path.isfile(pypifile) or not os.path.isfile(stdlibfile):
        print('You need to run "pipsalabim update" before trying to make a'
              ' report.')
        return 1

    if not os.path.isfile(os.path.join(basedir, 'setup.py')):
        print('Pip Sala Bim doesn\'t support folders without a setup.py.')
        return 1

    datadict = get_module_datadict(basedir)
    datadict = ask_multiple_pypi(datadict)
    messages = get_messages(datadict)

    if requirements:
        print('\nrequirements.txt file contents below')
        print('{0}>8{0}'.format('-' * 40))
        print('\n# file generated by Pip Sala Bim {0}'.format(__url__))
        print('\n'.join(m.split(':')[1] for m in messages['p']))
        return 0

    print('=' * 19)
    print('Pip Sala Bim Report')
    print('=' * 19)

    for msgtype, msgcont in messages.items():
        if not msgcont:
            continue
        if msgtype == 'l':
            print('\nThe folowing module imports have been found in your'
                  ' local source code:')
            print('\n'.join('    - {0}'.format(m) for m in msgcont))

        elif msgtype == 's':
            print('\nThe folowing module imports are part of python'
                  ' standard library:')
            for msg in msgcont:
                _mod, _py = msg.split(':')
                _py = _py.replace('STDLIB', '').split(',')
                _py = '{0} and {1}'.format(', '.join(_py[:-1]), _py[-1])
                print('    - {0} (python {1})'.format(_mod, _py))

        elif msgtype == 'p':
            print('\nThe folowing module imports where found in the PyPI'
                  ' module index:')
            print('\n'.join('    - {0} (available in "{1}" PyPI package. Use '
                            '"pip install {1}"'
                            ')'.format(*m.split(':')) for m in msgcont))
        elif msgtype == 'n':
            print('\nThe folowing module imports couldn\'t be found:')
            print('\n'.join('    - {0}'.format(m) for m in msgcont))

    if not messages['n'] and not messages['p']:
        print('\nCongratulations! All your imports are satisfied by the'
              ' python standard library or internal code.')

    return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import unittest
import doctest


def load_tests(loader, tests, pattern):
    tests.addTests(doctest.DocTestSuite('pipsalabim.core.util'))
    return tests

if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import doctest
import unittest

from pipsalabim.core.logger import logger


class TestLogger(unittest.TestCase):

    def setUp(self):
        logger.start()

    def test_01_default_level(self):
        pass


def load_tests(loader, tests, pattern):
    tests.addTests(doctest.DocTestSuite('pipsalabim.core.logger'))
    return tests

if __name__ == '__main__':
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.core.imports`` studies the code searching for imports.

This module has advanced programming to inspect and locate imported
modules in python source code. It uses python's own Abstract Syntax
Trees (AST) to do such operations.
"""
from __future__ import absolute_import, print_function

import ast

from .logger import logger


class ImportVisitor(ast.NodeVisitor):
    """
    AST visitor for grabbing the import statements.

    ``ImportVisitor`` is a node visitor class that walks the abstract syntax
    tree and calls a visitor function for every import statement found.

    Per default the visitor functions for the nodes are ``'visit_'`` +
    class name of the import node (lowercase). This visitor only has methods
    for processing ``Import`` and ``ImportFrom`` types of nodes. When any other
    type of node is entered, the `generic_visit` visitor is used instead.

    This visitor produces a list of tuples like::

        [
            ('MODULE', 'LEVEL'),
            ('MODULE', 'LEVEL'),
            ('MODULE', 'LEVEL'),
            ('MODULE', 'LEVEL'),
            ('MODULE', 'LEVEL')
        ]

    * ``MODULE``: a string containing the name of the imported module
                 (does not include the package to which it belongs).
    * ``LEVEL`` is the relative level of the imported module.

        - 0 is the level of a simple import like::

            import MODULE
            from MODULE import NAME

        - 1 is the level of a relative import in the same package,
          for example::

            import .MODULE
            from .MODULE import NAME
            from . import NAME

        - 2 is the level of a relative import in the parent package,
          for example::

            import ..MODULE
            from ..MODULE import NAME
            from .. import NAME
    """

    def __init__(self):
        """
        Initialize this ``ImportVisitor``.

        Sets initial empty values for ``self.modules`` which will help
        storing modules.

        :return: an ``ImportVisitor`` instance.

        .. versionadded:: 0.1.0
        """
        #: Attribute ``modules`` (list): Stores modules as they are found by
        #: the visitor methods.
        self.modules = []

    def visit(self, node):
        """
        Visit a node.

        :param node: an ``ast`` object representing a python statement.
        :return: a reference to ``visit_import`` if the node type is
                 ``Import``, to ``visit_importfrom`` if the node type is
                 ``ImportFrom`` or ``generic_visit`` for any other node.

        .. versionadded:: 0.1.0
        """
        if node.__class__.__name__ == 'Import':
            return self.visit_import(node)
        if node.__class__.__name__ == 'ImportFrom':
            return self.visit_importfrom(node)
        try:
            return self.generic_visit(node)
        except:
            pass

    def visit_import(self, node):
        """
        Append node names to ``self.modules``.

        :param node: an ``ast`` object representing a python statement.

        .. versionadded:: 0.1.0
        """
        self.modules.extend((n.name, 0) for n in node.names)

    def visit_importfrom(self, node):
        """
        Append node names and levels to ``self.modules``.

        :param node: an ``ast`` object representing a python statement.

        .. versionadded:: 0.1.0
        """
        if node.module != '__future__':
            self.modules.append((node.module, node.level))


def parse_python_source(filename):
    """
    Parse the file ``filename`` and convert it to AST.

    :param filename: a string containing a path to a python source code file.
    :return: an AST object or ``None`` if the file has a syntax error.

    .. versionadded:: 0.1.0
    """
    try:
        with open(filename, 'rU') as p:
            return ast.parse(p.read())
    except Exception as e:
        logger.error('Error processing file "{0}": {1}'.format(filename, e))
        return None


def find_imports(package, filename):
    """
    Get a list of modules extracted from import statements.

    :param package: a string containing a python package to which ``filename``
                    belongs.
    :param filename: a string containing a path to a python source code file.
    :return: a list of modules in absolute form.

    .. versionadded:: 0.1.0
    """
    imports = []
    tree = parse_python_source(filename)
    visitor = ImportVisitor()
    visitor.visit(tree)

    for modname, level in visitor.modules:
        if not modname:
            modname = ''
        if level == 1:
            modname = '{0}.{1}'.format(package, modname)
        if level == 2:
            modname = '{0}.{1}'.format('.'.join(package.split('.')[:-1]),
                                       modname)
        imports.append(modname.strip('.'))
    return imports



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.api.update`` is a module implementing the update command.

Pip Sala Bim needs to keep a local copy of the PyPIContents database
to function properly. This command takes care of maintaining it
up-to-date.

Please note that this command must be executed manually.
"""
from __future__ import absolute_import, print_function

import os

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from .. import stdliburl, pypiurl, stdlibfile, pypifile
from ..core.logger import logger
from ..core.util import chunk_report, chunk_read


def download_json(datafile, dataurl):
    """
    Download a json file from ``dataurl``, store to ``datafile`` and report.

    :param datafile: a string containing a path to store the contents of
                     the file downloaded from ``dataurl``.
    :param dataurl: a string containing a url to a file.
    :return: ``True`` if operations went OK, ``False`` if not.

    .. versionadded:: 0.1.0
    """
    if not os.path.isdir(os.path.dirname(datafile)):
        os.makedirs(os.path.dirname(datafile))

    print(('{0} is not present, downloading '
           '...').format(os.path.basename(datafile)))

    try:
        response = urlopen(url=dataurl, timeout=10)
        content = chunk_read(response, report_hook=chunk_report)
    except Exception as e:
        logger.error('Download error: {0}'.format(e))
        return False

    try:
        with open(datafile, 'w') as s:
            s.write(content.decode('utf-8'))
    except Exception as e:
        logger.error('I/O error: {0}'.format(e))
        return False
    return True


def main(**kwargs):
    """
    Update databases from PyPIContents.

    .. _stdlib.json: https://git.io/vXF1H
    .. _pypi.json: https://git.io/vXFDL
    .. _PyPIContents project: https://github.com/LuisAlejandro/pypicontents

    This function downloads the standard library modules index (`stdlib.json`_)
    and the PyPI modules index (`pypi.json`_) from the `PyPIContents project`_
    to keep a local copy of these files, which are critical to PiP Sala Bim.

    This command makes no comparing or checking prior to download.

    :return: an exit status.

    .. versionadded:: 0.1.0
    """
    print('Updating the standard library database ...')
    if download_json(stdlibfile, stdliburl):
        print('Success!\n')
    else:
        print('There was an error!\n')

    print('Updating the PyPIContents database ...')
    if download_json(pypifile, pypiurl):
        print('Success!\n')
    else:
        print('There was an error!\n')

    return 0



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.cli`` is a module for handling the command line interface.

This module handles the commands for using Pip Sala Bim. It also parses
parameters, show help, version and controls the logging level.
"""
from __future__ import absolute_import, print_function

from argparse import ArgumentParser

from . import __version__, __description__
from .core.logger import logger
from .api.report import main as report
from .api.update import main as update


def commandline(argv=None):
    """
    Configure ``ArgumentParser`` to accept custom arguments and commands.

    :param argv: a list of commandline arguments like ``sys.argv``.
                 For example::

                    ['-v', '--loglevel', 'INFO']

    :return: a ``Namespace`` object.

    .. versionadded:: 0.1.0
    """
    assert isinstance(argv, (list, type(None)))

    parser = ArgumentParser(description=__description__)
    parser.add_argument(
        '-V', '--version', action='version',
        version='pipsalabim {0:s}'.format(__version__),
        help='Print version and exit.')
    parser.add_argument(
        '-l', '--loglevel', default='WARNING', metavar='<level>',
        choices=['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'],
        help='Logger verbosity level (default: WARNING).')

    subparsers = parser.add_subparsers(title='commands')
    report_parser = subparsers.add_parser('report')
    report_parser.set_defaults(command=report)
    report_parser.add_argument(
        '-r', '--requirements', action='store_true',
        help='Format output for requirements.txt file.')

    update_parser = subparsers.add_parser('update')
    update_parser.set_defaults(command=update)

    return parser.parse_args(argv)


def main(argv=None):
    """
    Handle arguments and commands.

    :param argv: a list of commandline arguments like ``sys.argv``.
                 For example::

                    ['-v', '--loglevel', 'INFO']

    :return: an exit status.

    .. versionadded:: 0.1.0
    """
    assert isinstance(argv, (list, type(None)))

    args = commandline(argv)

    logger.start()
    logger.loglevel(args.loglevel)
    logger.info('Starting execution.')
    logger.debug('Processed arguments: {:s}'.format(vars(args)))

    try:
        status = args.command(**vars(args))
    except KeyboardInterrupt:
        logger.critical('Execution interrupted by user!')
        status = 1
    except Exception as e:
        logger.exception(e)
        logger.critical('Shutting down due to fatal error!')
        status = 1
    else:
        logger.info('Ending execution.')

    logger.stop()
    return status


if __name__ == '__main__':
    import re
    import sys
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit(main())



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.core`` contains the core functions of the application.

This package contains functions for managing the download of files,
exploring and obtaining imported modules, logging and other minor
file operations.
"""



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim`` is just black magic.

Pip Sala Bim is a package that studies the codebase of your project in search
for internal and external imports. It then discards the imports that are
satisfied with internal code or with the standard library and finally
searches the `PyPIContents`_ index to list which packages satisfy your imports.

.. _PyPIContents: https://github.com/LuisAlejandro/pypicontents

"""
import os
from distutils import sysconfig


__author__ = 'Luis Alejandro Martínez Faneyth'
__email__ = 'luis@huntingbears.com.ve'
__version__ = '0.1.3'
__url__ = 'https://github.com/LuisAlejandro/pipsalabim'
__description__ = ('Pip Sala Bim is an assistant to guess your pip'
                   ' dependencies from your code, without using a'
                   ' requirements file.')
stdliburl = ('https://raw.githubusercontent.com/LuisAlejandro/pypicontents/'
             'contents/stdlib.json')
pypiurl = ('https://raw.githubusercontent.com/LuisAlejandro/pypicontents/'
           'contents/pypi.json')
stdlibfile = os.path.join(os.environ.get('HOME', os.path.expanduser('~')),
                          '.cache', 'pipsalabim', 'stdlib.json')
pypifile = os.path.join(os.environ.get('HOME', os.path.expanduser('~')),
                        '.cache', 'pipsalabim', 'pypi.json')
libdir = sysconfig.get_python_lib(standard_lib=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from pipsalabim import (__author__, __email__, __version__, __url__,
                        __description__)

setup(
    name='pipsalabim',
    version=__version__,
    author=__author__,
    author_email=__email__,
    url=__url__,
    description=__description__,
    long_description=open('README.rst').read(),
    packages=['pipsalabim', 'pipsalabim.api', 'pipsalabim.core'],
    package_dir={'pipsalabim': 'pipsalabim'},
    include_package_data=True,
    install_requires=open('requirements.txt').read().split('\n'),
    entry_points={
        'console_scripts': ('pipsalabim = pipsalabim.cli:main',),
    },
    license=open('COPYING.rst').read(),
    zip_safe=False,
    keywords=['pip', 'requirements.txt', 'guess', 'report'],
    platforms=['posix', 'linux'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=open('requirements-dev.txt').read().split('\n')
)



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.core.utils`` is a utility module.

This module contains several utilities to process information coming from the
other modules.
"""
from __future__ import absolute_import, print_function

import os
import sys
import fnmatch
from contextlib import contextmanager

try:
    basestring
except NameError:
    basestring = str


@contextmanager
def custom_sys_path(new_sys_path):
    """
    Context manager to momentarily change ``sys.path``.

    :param new_sys_path: a list of paths to overwrite ``sys.path``.

    .. versionadded:: 0.1.0
    """
    old_sys_path = sys.path
    sys.path = new_sys_path
    yield
    sys.path = old_sys_path


@contextmanager
def remove_sys_modules(remove):
    """
    Context manager to momentarily remove modules from ``sys.modules``.

    :param remove: a list of modules to remove from ``sys.modules``.

    .. versionadded:: 0.1.0
    """
    old_sys_modules = sys.modules
    for r in remove:
        if r in sys.modules:
            del sys.modules[r]
    yield
    sys.modules = old_sys_modules


def list_files(path=None, pattern='*'):
    """
    List files on ``path`` (non-recursively).

    Locate all the files matching the supplied filename pattern in the first
    level of the supplied ``path``. If no pattern is supplied, all files will
    be returned.

    :param path: a string containing a path where the files will be looked for.
    :param pattern: a string containing a regular expression.
    :return: a list of files matching the pattern within the first level of
             path (non-recursive).

    .. versionadded:: 0.1.0
    """
    assert isinstance(path, basestring)
    assert isinstance(pattern, basestring)

    filelist = []
    for f in fnmatch.filter(os.listdir(path), pattern):
        if os.path.isfile(os.path.join(path, f)):
            filelist.append(os.path.join(path, f))
    return filelist


def find_files(path=None, pattern='*'):
    """
    Locate all the files matching the supplied ``pattern`` in ``path``.

    Locate all the files matching the supplied filename pattern in and below
    the supplied root directory. If no pattern is supplied, all files will be
    returned.

    :param path: a string containing a path where the files will be looked for.
    :param pattern: a string containing a regular expression.
    :return: a list of files matching the pattern within path (recursive).

    .. versionadded:: 0.1
    """
    assert isinstance(path, basestring)
    assert isinstance(pattern, basestring)

    filelist = []
    for directory, subdirs, files in os.walk(os.path.normpath(path)):
        for filename in fnmatch.filter(files, pattern):
            if os.path.isfile(os.path.join(directory, filename)):
                filelist.append(os.path.join(directory, filename))
    return filelist


def is_valid_path(path):
    """
    Test if ``path`` is a valid python path.

    :param path: a string containing a path.
    :return: ``True`` if ``path`` is a valid python path. ``False``
             otherwise.

    .. versionadded:: 0.1.0
    """
    for component in os.path.normpath(path).split(os.sep):
        if ('.' in component or '-' in component) and \
           component not in ['.', '..']:
            return False
    return True


def chunk_report(downloaded, total):
    """
    Print the progress of a download.

    :param downloaded: an integer representing the size (in bytes) of data
                       downloaded so far.
    :param total: an integer representing the total size (in bytes) of data
                  that needs to be downloaded.

    .. versionadded:: 0.1.0
    """
    percent = round((float(downloaded) / total) * 100, 2)
    sys.stdout.write(('Downloaded {:d} of {:d} kB '
                      '({:0.2f}%)\r').format(downloaded / 60,
                                             total / 60, percent))
    if downloaded >= total:
        sys.stdout.write('\n\n')


def chunk_read(response, chunk_size=8192, report_hook=None):
    """
    Download a file by chunks.

    :param response: a file object as returned by ``urlopen``.
    :param chunk_size: an integer representing the size of the chunks to be
                       downloaded at a time.
    :param report_hook: a function to report the progress of the download.
    :return: a blob containing the downloaded file.

    .. versionadded:: 0.1.0
    """
    data = []
    downloaded = 0
    total = int(response.info().getheader('Content-Length').strip())

    while True:
        chunk = response.read(chunk_size)

        if not chunk:
            break

        data += chunk
        downloaded += len(chunk)

        if report_hook:
            report_hook(downloaded, total)

    return ''.join(data)


def fill_with_local(datadict, modules):
    """
    Fill ``datadict`` if module is found in ``modules``.

    :param datadict: a dictionary containing modules as keys and
                     a list as values.
    :param modules: a list of modules present in the local python source code.
    :return: a dictionary containing information about the location of each
             imported module.

    .. versionadded:: 0.1.0
    """
    for module, where in datadict.items():
        if not where and module in modules:
            datadict[module].append('LOCAL')
    return datadict


def fill_with_stdlib(datadict, stdlibdata):
    """
    Fill ``datadict`` with modules from ``stdlibdata`` if found.

    :param datadict: a dictionary containing modules as keys and
                     a list as values.
    :param stdlibdata: a dictionary containing the modules of the standard
                       library of each python version.
    :return: a dictionary containing information about the location of each
             imported module.

    .. versionadded:: 0.1.0
    """
    for module, where in datadict.items():
        if where:
            continue
        for version, mods in stdlibdata.items():
            if module not in mods:
                continue
            datadict[module].append('STDLIB{0}'.format(version))
    return datadict


def fill_with_pypi(datadict, pypidata):
    """
    Fill ``datadict`` with modules from ``pypidata`` if found.

    .. _PyPIContents: https://github.com/LuisAlejandro/pypicontents

    :param datadict: a dictionary containing modules as keys and
                     a list as values.
    :param pypidata: a dictionary with the `PyPIContents`_ database.
    :return: an updated dictionary containing information about the location
             of each imported module.

    .. versionadded:: 0.1.0
    """
    for module, where in datadict.items():
        if where:
            continue
        for package, data in pypidata.items():
            if module not in data['modules']:
                continue
            datadict[module].append(package)
    return datadict



# -*- coding: utf-8 -*-
#
#   This file is part of Pip Sala Bim.
#   Copyright (C) 2016, Pip Sala Bim Developers.
#
#   Please refer to AUTHORS.rst for a complete list of Copyright holders.
#
#   Pip Sala Bim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Pip Sala Bim is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses.
"""
``pipsalabim.core.logger`` is the global application logging module.

All modules use the same global logging object. No messages will be emitted
until the logger is started.
"""
from __future__ import absolute_import, print_function

import types
import logging


class ControlableLogger(logging.Logger):
    """
    This class represents a logger object that can be started and stopped.

    It has a start method which allows you to specify a logging level. The stop
    method halts output.
    """

    def __init__(self, name=None):
        """
        Initialize this ``ControlableLogger``.

        The name defaults to the application name. Loggers with the same name
        refer to the same underlying object. Names are hierarchical, e.g.
        'parent.child' defines a logger that is a descendant of 'parent'.

        :param name: a string containig the logger name.
        :return: a ``ControlableLogger`` instance.

        .. versionadded:: 0.1.0
        """
        # Initializing according to old-style or new-style clases
        if hasattr(types, 'ClassType') and \
           isinstance(logging.Logger, types.ClassType):
            logging.Logger.__init__(self, __name__.split('.')[0])
        if (hasattr(types, 'TypeType') and
           isinstance(logging.Logger, types.TypeType)) or \
           isinstance(logging.Logger, type):
            super(ControlableLogger, self).__init__(__name__.split('.')[0])

        #: Attribute ``active`` (boolean): Stores the current status of the
        #: logger.
        self.active = False

        #: Attribute ``formatstring`` (string): Stores the string that
        #: will be used to format the logger output.
        self.formatstring = '%(levelname)s: %(message)s'

    def start(self):
        """
        Start logging with this logger.

        Until the logger is started, no messages will be emitted. This applies
        to all loggers with the same name and any child loggers.

        .. versionadded:: 0.1.0
        """
        if not self.active:
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter(self.formatstring))
            self.addHandler(handler)
            self.active = True

    def stop(self):
        """
        Stop logging with this logger.

        Remove available handlers and set active attribute to ``False``.

        .. versionadded:: 0.1.0
        """
        if self.active:
            self.removeHandler(self.handlers[-1])
            self.active = False

    def loglevel(self, level='WARNING'):
        """
        Set the log level for this logger.

        Messages less than the given priority level will be ignored. The
        default level is 'WARNING', which conforms to the *nix convention that
        a successful run should produce no diagnostic output. Available levels
        and their suggested meanings:

        * ``NOTSET``: all messages are processed.
        * ``DEBUG``: output useful for developers.
        * ``INFO``: trace normal program flow, especially external
                    interactions.
        * ``WARNING``: an abnormal condition was detected that might need
                       attention.
        * ``ERROR``: an error was detected but execution continued.
        * ``CRITICAL``: an error was detected and execution was halted.

        :param level: a string containing the desired logging level.

        .. versionadded:: 0.1.0
        """
        if self.active:
            self.setLevel(level)


logger = ControlableLogger()
